/*
** dcc.c (for irchat-2.4jp)
** Copyright (C) 1995, 1996 Takahiro Kikuchi
**
** Author:        Takahiro Kikuchi <kick@kyoto.wide.ad.jp>
** Created:       Mar 19, 1995
** Last Modified: Jul 19, 1996
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <netinet/in.h>
#include <stdio.h>
#include <fcntl.h>
#include <netdb.h>
#ifdef	AIX
#include <sys/select.h>
#endif
#ifdef linux
#ifndef __GLIBC__
#include <linux/time.h>
#endif
#endif
int main(argc, argv)
int argc;
char **argv;
{
    char *command, *type, *action;
    int status = 1;
    extern u_long atoul(), dottoul();
    
    command = *argv++, argc--;
    if (argc < 2) {
	printf("DCC ERROR Not enough parameters\n");
	fprintf(stderr, "Usage: %s <type> <command> <arg>...\n", command);
	exit(98);
    }

    type = *argv++, argc--;
    action = *argv++, argc--;

    if (!strcmp(type, "file")) {
	if (!strcmp(action, "send")) {
	    if (argc != 1) {
		printf("DCC ERROR Wrong number of parameters\n");
		fprintf(stderr,
			"Usage: %s file send <filename>\n", command);
		status = 98;
	    } else {
		status = file_send(argv[0]);
	    }
	} else if (!strcmp(action, "get")) {
	    if (argc != 4) {
		printf("DCC ERROR Wrong number of parameters\n");
		fprintf(stderr,
			"Usage: %s file get <host> <port> <size> <filename>\n",
			command);
		status = 98;
	    } else {
		status = file_get(atoul(argv[0]), atoi(argv[1]),
				  atoi(argv[2]), argv[3]);
	    }
	} else {
	    printf("DCC ERROR Unsupported command %s %s\n", type, action);
	    status = 99;
	}
    } else if (!strcmp(type, "chat")) {
	if (!strcmp(action, "listen")) {
	    if (argc != 0) {
		printf("DCC ERROR Wrong number of parameters\n");
		fprintf(stderr, "Usage: %s chat listen\n", command);
		status = 98;
	    } else {
		status = chat_listen();
	    }
	} else if (!strcmp(action, "connect")) {
	    if (argc != 2) {
		printf("DCC ERROR Wrong number of parameters\n");
		fprintf(stderr, "Usage: %s chat connect <host> <port>\n",
			command);
		status = 98;
	    } else {
		status = chat_connect(atoul(argv[0]), atoi(argv[1]));
	    }
	} else {
	    printf("DCC ERROR Unsupported command %s %s\n", type, action);
	    status = 99;
	}
    } else if (!strcmp(type, "tcp")) {
	if (!strcmp(action, "connect")) {
	    if (argc != 2) {
		printf("DCC ERROR Wrong number of parameters\n");
		fprintf(stderr, "Usage: %s chat connect <host> <port>\n",
			command);
		status = 98;
	    } else {
		status = tcp_connect(dottoul(argv[0]), atoi(argv[1]));
	    }
	} else {
	    printf("DCC ERROR Unsupported command %s %s\n", type, action);
	    status = 99;
	}
    } else {
	printf("DCC ERROR Unsupported command %s\n", type);
	status = 99;
    }
    if (status) {
	sleep(3);
    }
    exit(status);
}

int dcc_listen(phost, pport)
u_long *phost;
int *pport;
{
    int i, s, len;
    u_long host;
    char hostname[MAXHOSTNAMELEN];
    struct hostent *hp;
    struct sockaddr_in server;

    gethostname(hostname, sizeof(hostname));
    if ((hp = gethostbyname(hostname)) == NULL) {
	printf("DCC ERROR Cannot gethostbyname %s\n", hostname);
	return -1;
    }
    host = ntohl(((struct in_addr *) hp->h_addr_list[0])->s_addr);

    if ((s = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
	printf("DCC ERROR Cannot create socket\n");
	return -1;
    }
    server.sin_family = AF_INET;
    server.sin_addr.s_addr = htonl(host);
    server.sin_port = 0;
    for (i = 0; i < 8; i++) {
	server.sin_zero[i] = 0;
    }
    if (bind(s, (struct sockaddr *)&server, sizeof(server)) < 0) {
	printf("DCC ERROR Cannot bind socket\n");
	return -1;
    }
    if (listen(s, 1) < 0) {
	printf("DCC ERROR Cannot listen socket\n");
	return -1;
    }
    len = sizeof(server);
    if (getsockname(s, (struct sockaddr *)&server, &len) < 0) {
	printf("DCC ERROR Cannot getsockname\n");
	return -1;
    }	
    *pport = ntohs(server.sin_port);
    *phost = ntohl(server.sin_addr.s_addr);

    return s;
}

int dcc_connect(host, port)
u_long host;
int port;
{
    int i, remote;
    struct sockaddr_in server;

    if ((remote = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
	printf("DCC ERROR Cannot create socket\n");
	return -1;
    }
    server.sin_family = AF_INET;
    server.sin_addr.s_addr = htonl(host);
    server.sin_port = htons(port);
    for (i = 0; i < 8; i++) {
	server.sin_zero[i] = 0;
    }
    if (connect(remote, (struct sockaddr *)&server, sizeof(server)) < 0) {
	printf("DCC ERROR Cannot connect %u/%d\n", host, port);
	return -1;
    }
    return remote;
}

u_long atoul(str)
char *str;
{
    u_long val = 0;

    while (*str) {
	val = val * 10 + *str - '0';
	str++;
    }
    return val;
}

u_long dottoul(str)
char *str;
{
    u_long val, v;

    for (val = v = 0; *str; str++) {
	if (*str == '.') {
	    val = val * 256 + v;
	    v = 0;
	} else {
	    v = v * 10 + *str - '0';
	}
    }
    val = val * 256 + v;
    return val;
}

int file_send(filename)
char *filename;
{
    int file, size, s, remote, port, len, count = 0;
    u_long host;
    struct stat statbuf;
    u_long done = 0;
    u_long report;
    char buf[4096];

    if ((file = open(filename, O_RDONLY)) < 0) {
	printf("DCC ERROR Cannot open file %s\n", filename);
	return 1;
    }
    if (fstat(file, &statbuf) < 0) {
	printf("DCC ERROR Cannot stat file %s\n", filename);
	return 1;
    }
    size = statbuf.st_size;

    if ((s = dcc_listen(&host, &port)) < 0) {
	/* error report in dcc_listen */
	return 1;
    }
    /* 'Setting -> 'Waiting (send DCC SEND to remote user) */
    printf("DCC SEND %u %d %d\n", host, port, size);

    remote = accept(s, (struct sockaddr *) 0, (int *) 0);
    close(s);
    /* 'Waiting -> 'Sending (accepted from remote user) */
    printf("DCC SENDING\n");

    while ((len = read(file, buf, sizeof(buf))) > 0) {
	write(remote, buf, len);
	done += len;
	if (++count == 16) {
	    count = 0;
	    while (read(remote, &report, sizeof(u_long)) > 0 &&
		   ntohl(report) != done);
	    printf("DCC REPORT %s %d%% (%d/%d bytes) sent\n",
		   filename, 100 * done / size, done, size);
	}
    }
    while (read(remote, &report, sizeof(u_long)) > 0 &&
	   ntohl(report) != done);

    /* 'Sending -> end */
    close(remote);
    close(file);
    return 0;
}

int file_get(host, port, size, filename)
u_long host;
int port;
int size;
char *filename;
{
    int remote, file, len, toread, count = 0;
    u_long done = 0;
    u_long report;
    char buf[4096];

    if ((file = open(filename, O_WRONLY|O_CREAT|O_TRUNC, 0600)) < 0) {
	printf("DCC ERROR1 Cannot open file %s\n", filename);
	return 1;
    }

    if ((remote = dcc_connect(host, port)) < 0) {
	/* error report in dcc_connect */
	return 1;
    }
    /* 'Connect -> 'Getting (connected to remote user) */
    printf("DCC GETTING\n");

    toread = sizeof(buf);
    while (size - done > 0) {
	if (toread > size - done) {
	    toread = size - done;
	}
	if ((len = read(remote, buf, toread)) < 0) {
	    printf("DCC ERROR read error %s\n", filename);
	    return 1;
	}
	write(file, buf, len);
	done += len;
	report = htonl(done);
	write(remote, &report, sizeof(report));
	if (++count == 16) {
	    count = 0;
	    printf("DCC REPORT %s %d%% (%d/%d bytes) received\n",
		   filename, 100 * done / size, done, size);
	}
    }

    /* 'Getting -> end */
    close(remote);
    close(file);
    return 0;
}

int chat_listen()
{
    u_long host;
    int port;
    int s, remote;

    if ((s = dcc_listen(&host, &port)) < 0) {
	return 1;
    }
    /* 'Setting -> 'Waiting (send DCC CHAT to remote user) */
    printf("DCC CHAT %u %d\n", host, port);

    remote = accept(s, (struct sockaddr *) 0, (int *) 0);
    close(s);
    /* 'Waiting -> 'Active (accepted from remote user) */
    printf("DCC CHATTING\n");
    return chat_loop(remote);
}

int chat_connect(host, port)
u_long host;
int port;
{
    int remote;

    if ((remote = dcc_connect(host, port)) < 0) {
	return 1;
    }
    /* 'Connect -> 'Active (connected to remote user) */
    printf("DCC CHATTING\n");
    return chat_loop(remote);
}

int chat_loop(remote)
int remote;
{
    int n, len, cnt;
    char *ptr, buf[1024];
    fd_set rfds;

    while (1) {
	FD_ZERO(&rfds);
	FD_SET(0, &rfds);
	FD_SET(remote, &rfds);
	if ((n = select(FD_SETSIZE, &rfds, NULL, NULL, NULL)) == -1) {
	    continue;
	}
	if (FD_ISSET(0, &rfds)) {
	    if ((len = read(0, buf, sizeof(buf))) <= 0) {
		close(0);
		close(remote);
		if (len == 0) {
		    return 0;
		} else {
		    return 5;
		}
	    }
	    for (ptr = buf; len > 0; ptr+= cnt, len -= cnt) {
		if ((cnt = write(remote, ptr, len)) < 0) {
		    close(0);
		    close(remote);
		    return 6;
		}
	    }
	}
	if (FD_ISSET(remote, &rfds)) {
	    if ((len = read(remote, buf, sizeof(buf))) <= 0) {
		close(0);
		close(remote);
		if (len == 0) {
		    return 0;
		} else {
		    return 7;
		}
	    }
	    for (ptr = buf; len > 0; ptr+= cnt, len -= cnt) {
		if ((cnt = write(0, ptr, len)) < 0) {
		    close(0);
		    close(remote);
		    return 8;
		}
	    }
	}
    }
}

int tcp_connect(host, port)
u_long host;
int port;
{
    int remote, status;

    if ((remote = dcc_connect(host, port)) < 0) {
	return 97;
    }
    if (status = chat_loop(remote)) {
	return 1;
    } else {
	return 1;
    }
}

