;;;
;;; $Id: irchat-filter.el,v 1.5 1993/10/07 12:44:59 tmo Exp $
;;;
;;; see file irchat-copyright.el for change log and copyright info

(require 'defsubst)
(provide 'irchat-filter)

;;;
;;;  These are defsubst just for speed, as it is expensive to call funtions at
;;;  emacs lisp (also evals are expensive)
;;;
(defsubst irchat-current-minute ()
  (string-to-int (substring (current-time-string) 14 16)))
(defvar irchat-minute 0)

(defun irchat-handle (line)
  "Called when we have at least one line of output from the IRC server."
  (let ((obuf (current-buffer))
	prefix userhost (cmd "") arg)
    (while (string-match "^\\([^\r]*\\)\r+\\(.*\\)" line)
      (setq line (concat (matching-substring line 1)
			 (matching-substring line 2))))
    (if (string-match "^:\\([^ ]+\\) *\\(.*\\)" line)
	(setq prefix (matching-substring line 1)
	      line (matching-substring line 2)))
    (if (and prefix (string-match "^\\([^!]*\\)!\\(.*\\)" prefix))
	(setq userhost (matching-substring prefix 2)
	      prefix (matching-substring prefix 1)))
    (if (string-match "^ *\\([^ ]+\\) *\\(.*\\)" line)
	(setq arg (list (matching-substring line 1))
	      line (matching-substring line 2)))
    (while (string-match "^\\([^:][^ ]*\\) +\\(.*\\)" line)
      (nconc arg (list (matching-substring line 1)))
      (setq line (matching-substring line 2)))
    (if (not (string= line ""))
	(if (string-match "^:\\(.*\\)" line)
	    (nconc arg (list (matching-substring line 1)))
	  (nconc arg (list line))))
    (if (and prefix userhost
	     (not (string= (get (intern prefix) 'userhost) userhost)))
	(put (intern prefix) 'userhost userhost))
    (if (car arg)
	(setq cmd (downcase (car arg))
	      arg (cdr arg)))
    (if (and (boundp (setq hook
			   (intern (concat "irchat-handle-" cmd "-hook"))))
	     (eval hook)
	     (eval (list (eval hook) prefix arg)))
	nil
      (setq fun (intern (concat "irchat-handle-" cmd)))
      (if (fboundp fun)
	  (apply fun prefix arg)
	(if irchat-debugging
	    (irchat-insert-special (format "No handle-%s (%s) %s\n" cmd prefix
					   (prin1-to-string arg))))
	(let* ((cmd-number (string-to-int cmd))
	       (default-number (/ cmd-number 100)) tmp)
	  (setq fun (intern (concat "irchat-handle-"
				    (format "%d00s" default-number))))
	  (if (and (> cmd-number 0)
		   (fboundp fun))
	      (apply fun cmd-number prefix arg)
	    (setq tmp (format "Unknown MESSAGE: [%s] [%s] [%s]\n" 
				    prefix cmd (prin1-to-string arg)))
	    (message tmp)
	    (irchat-insert0 tmp)))))
    (set-buffer obuf)))

(defun irchat-pj-filter (process output)
  "Filter function for IRC server process (preprocess).
Code conversion and call original irchat-filter."
  (if (and (boundp 'irchat-pj-filter-level) (<= irchat-pj-filter-level 2))
      (irchat-filter process output)
    (let (tmp x real-output code)
      (if (null (boundp 'irchat-pj-filter-queue))
	  (setq irchat-pj-filter-queue ""))
      (setq tmp (concat irchat-pj-filter-queue output))
      (while (setq x (string-match "\012" tmp))
	(setq x (1+ x))
	(setq real-output (substring tmp 0 x))
	(setq tmp (substring tmp x))
	;; decide/convert code
	(setq code (irchat-pj-filter-decide-code real-output))
	(if (boundp 'irchat-pj-filter-level)
	    (cond ((or (= irchat-pj-filter-level 3) (= irchat-pj-filter-level 4))
		   (if (null (string= "JIS" code))
		       (setq code "NOCONV")))
		  ((= irchat-pj-filter-level 5)
		   (if (null (or (string= "JIS" code) (string= "NOCONV" code)))
		       (setq code "SJIS")))))
	(if (and irchat-pj-code-verbose (or (string= "EUC" code) (string= "SJIS" code)))
	    (message "%s code appeared." code))
	(setq real-output (irchat-pj-filter-convert-code code real-output))
	;; real filter routine
	(irchat-filter process real-output))
      (setq irchat-pj-filter-queue tmp))))

;; devide decide-code-part of filter function
;; Sun, Sep 27, 1998 by simm-irc@fan.gr.jp

(defun irchat-pj-filter-decide-code (str)
  "Decide code of argument string."
  (let ((len (length str))(pp 0)(ps 0)(pe 0)(fl 0)(code nil))
    (while (null code)
      (setq ps (elt str pp))
      (setq pp (1+ pp))
      (if (= pp len)
	  (setq code "-"))
      (cond ((= 27 ps)                     ;; ESC
	     (setq code "JIS"))
	    ((or (= 13 ps) (= 10 ps))      ;; CRLF
	     (setq code "-"))
	    ((and (<= 128 ps) (<= ps 159)) ;; 0x80 <= ch <= 0x9f
	     (setq code "SJIS"))
	    ((and (= pe 1) (<= ps 127))    ;; second byte MSB = 0
	     (setq code "SJIS"))
	    ((and (= 2 pe) (<= 161 ps) (<= ps 243)) ;; second byte of EUC kana
	     (setq fl 2)
	     (setq pe 0))
	    ((and (= 0 pe) (= 164 ps))     ;; first byte of EUC kana
	     (setq pe 2))
	    ((<= 160 ps)                   ;; 0xa0 <= ch
	     (if (null (= 2 fl ))
		 (setq fl 1))
	     (if (= 2 pe)
		 (setq code "SJIS")
	       (setq pe (- 1 pe))))))
    (if (string= "-" code)
	(cond ((= 1 pe) (setq code "SJIS"))
	      ((= 2 pe) (setq code "SJIS"))
	      ((= 0 fl) (setq code "NOCONV"))
	      ((= 1 fl) (setq code "SJIS"))
	      ((= 2 fl) (setq code "EUC"))))
    code))

;; devide convert-code-part of filter function
;; Sun, Sep 27, 1998 by simm-irc@fan.gr.jp

(defun irchat-pj-filter-convert-code (code str)
  "Convert code of argument string."
  (let* ((pp 0)(ps 0)(pe 0)(len (length str)))
    (cond
     ;; round tail for JIS
     ((string= "JIS" code)
      (while (setq ps (string-match "\033\044B" str pp))
	(setq pe (string-match "\033(B" str ps))
	(if (null pe)
	    (progn
	      (setq len (1- len))
	      (if (= 10 (elt str len))
		  (if (= 13 (elt str (1- len)))
		      (setq len (1- len)))
		(setq len (1+ len)))
	      (setq str (concat (substring str 0 len) "\033(B\015\012"))
	      (setq pe (string-match "\033(B" str ps))))
	(if (eq 1 (% (- pe ps) 2))
	    (setq pp (+ 3 pe))
	  (setq str
		(concat (substring str 0 (- pe 1))
			(substring str pe)))
	  (setq pp (+ 2 pe))))
      ;;
      (cond ((fboundp 'code-convert-string)
	     (if (null (and irchat-emacs19 (fboundp 'skk-mode)))
		 (setq str (code-convert-string str *junet* *internal*))
	       (if (fboundp 'ad-deactivate)
		   (ad-deactivate 'kill-buffer))
	       (setq str (code-convert-string str *junet* *internal*))
	       (if (fboundp 'ad-activate)
		   (ad-activate 'kill-buffer))))
	    ((fboundp 'decode-coding-string)
	     (setq str (decode-coding-string str 'iso-2022-jp)))))
     ;; round tail for SJIS
     ((string= "SJIS" code)
      (cond ((fboundp 'code-convert-string)
	     (if (null (and irchat-emacs19 (fboundp 'skk-mode)))
		 (setq str (code-convert-string str *sjis* *internal*))
	       (if (fboundp 'ad-deactivate)
		   (ad-deactivate 'kill-buffer))
	       (setq str (code-convert-string str *sjis* *internal*))
	       (if (fboundp 'ad-activate)
		   (ad-activate 'kill-buffer))))
	    ((fboundp 'decode-coding-string)
	     (setq str (decode-coding-string str 'sjis)))))
     ;; round tail for EUC
     ((string= "EUC" code)
      (cond ((fboundp 'code-convert-string)
	     (if (null (and irchat-emacs19 (fboundp 'skk-mode)))
		 (setq str (code-convert-string str *euc-jp* *internal*))
	       (if (fboundp 'ad-deactivate)
		   (ad-deactivate 'kill-buffer))
	       (setq str (code-convert-string str *euc-jp* *internal*))
	       (if (fboundp 'ad-activate)
		   (ad-activate 'kill-buffer))))
	    ((fboundp 'decode-coding-string)
	     (setq str (decode-coding-string str 'euc-jp))))))
    (if (string-match "\015\012" str)
	(setq str (concat (substring str 0 (- (length str) 2)) "\012")))
    str))

;;

(defun irchat-filter (process output)
  "Filter function for IRC server process."
  (let ((obuf (current-buffer))
	(data (match-data))
	line)
    ;;
    ;; C-c D creates debug buffer for incoming messages...
    ;;
    (if (and irchat-debug-buffer (get-buffer irchat-debug-buffer))
	(progn
	  (set-buffer irchat-debug-buffer)
	  (let* ((dbgwin (get-buffer-window irchat-debug-buffer))
		 (wp (if dbgwin (window-point dbgwin) nil))
		 (pm (point-max)))
	    (save-excursion
	      (goto-char (point-max))
	      (insert output))
	    (if (and wp (>= wp pm))
		(irchat-scroll-if-visible dbgwin)))))

    (set-buffer (process-buffer process))
    (goto-char (point-max))
    (insert output)
    (goto-char (point-min))
    (while (looking-at ".*\n")
      (goto-char (point-min))
      (setq line (buffer-substring (match-beginning 0) (1- (match-end 0))))
      (delete-region (match-beginning 0) (match-end 0))
      (irchat-handle line))
    (if (/= (irchat-current-minute) irchat-minute)
	(let (fun)
	  (setq irchat-minute (irchat-current-minute))
	  (if (fboundp (setq fun (intern "irchat-minutely")))
	      (apply fun nil))))
    (set-buffer obuf)
    (store-match-data data)))

(defun irchat-sentinel (proc status)
  "Sentinel function for IRC server process."
  (if (and irchat-server-process
	   (not (irchat-server-opened)))
      (if (not (or irchat-reconnect-automagic irchat-reconnect-with-password))
	  (if irchat-fatal-error-message
	      (error (format "IRC ERROR: %s" irchat-fatal-error-message))
	    (if (process-id proc)
		(irchat-sentinel2 proc status)
	      (error (format "IRC ERROR: Connection closed. (%s)"
			     (substring status 0 (1- (length status)))))))
	(if (and irchat-grow-tail (not irchat-reconnect-with-password))
	    (progn
	      (setq irchat-nickname (concat irchat-nickname irchat-grow-tail))
	      (irchat 'always))
	  (irchat)))))

(defun irchat-sentinel2 (proc status)
  (if (not (string-match "^exited abnormally with code \\([0-9]+\\)" status))
      (error (format "IRC ERROR: Connection closed. (%s)"
		     (substring status 0 (1- (length status)))))
    (let ((status (string-to-int (matching-substring status 1))))
      (cond
       ((= 99 status) ;; unsupported command
	(error (format "IRC ERROR: Please use a newer \"%s\"."
		       irchat-dcc-program)))
       ((= 98 status) ;; bad argment number
	(error (format "IRC ERROR: Please use a newer \"%s\"."
		       irchat-dcc-program)))
       ((= 97 status)
	(error (format "IRC ERROR: Cannot connect to IRC server.")))
       (t
	(error (format "IRC ERROR: Server connection closed.")))))))

;;;
;;; eof
;;;
