;;;
;;; $Id: irchat-vars.el,v 1.4 1994/02/23 15:05:00 tmo Exp $
;;;
;;; see file irchat-copyright.el for change log and copyright info

(provide 'irchat-vars)

;;;
;;; user modifiable variables
;;;

(defvar irchat-command-window-height 4
  "*How large should Command window be on startup.")

(defvar irchat-channel-window-height-percent 63
  "*How large percent should Current channel window be on startup.")

(defvar irchat-want-traditional nil
  "*Do we want Irchat to look like IrcII.")

(defvar irchat-want-jp-compatible t
  "*Do we want this to look like irchat1.25jp")

(defvar irchat-variables-file "~/.irchat_vars.el"
  "*Where to look for variables. Helps to remove clutter from your .emacs.
This feature is most likely to dissappear in near future. The preferred 
way is to put irchat variables on .emacs or file loaded from there.")

(defvar irchat-server-alist
  '(("irc.kyoto.wide.ad.jp")
    ("irc.tokyo.wide.ad.jp") ("irc.race.u-tokyo.ac.jp"))
  "*IRC server assoc-list which is used for input IRC server.")

(defvar irchat-server (or (getenv "IRCSERVER") nil)
  "*Name of the host running the IRC server. 
Value initialized from the IRCSERVER environment variable if one is set")

(defvar irchat-service 
  (let ((ircport-env (getenv "IRCPORT")))
    (if ircport-env
	(if (> (string-to-int ircport-env) 0)
	    (string-to-int ircport-env)
	  ircport-env)
      6667))
  "*IRC service name or (port) number.")

(defvar irchat-dcc-auto-get-file nil
  "*If non-nil, get a file automatically when DCC request received.")

(defvar irchat-dcc-program "dcc"
  "*Name of dcc-program.")

(defvar irchat-dcc-directory "~/tmp"
  "*Directory where irchat-dcc puts its files.")

(defvar irchat-name (or (getenv "IRCNAME") (getenv "NAME") (user-full-name))
  "*Your realname.
Initialized from the IRCNAME or NAME environment variable, or your GCOS field.")

(defvar irchat-password (or (getenv "IRCPASSWORD") nil)
  "*Your password when connecting to server.")

(defvar irchat-nickname (or (getenv "IRCNICK") (user-real-login-name))
  "*The nickname you want to use in IRC.
Default is the environment variable IRCNICK, or your login name.")

(defvar irchat-servername "")

(defvar irchat-startup-channel nil
  "obsolete.")

(defvar irchat-startup-channel-list nil
  "*The channel list to join automatically at startup.
If nil, do not join any channel.")

(defvar irchat-default-channel-binding nil
  "*The channel list to bind the channel number when joining.")

(defvar irchat-channel-conversion-map nil
  "*The map of channel conversion.")

(defvar irchat-channel-conversion-default-mask "*.jp"
  "*The default mask of channel conversion.")

(defvar irchat-format-string ">%s<"
  "*Format string for private messages being sent.")

(defvar irchat-format-string1 "=%s="
  "*Format string for arriving private messages.")

(defvar irchat-format-string2 "<%s>"
  "*Format string for arriving messages to current channel.")

(defvar irchat-format-string3 "<%s:%s>"
  "*Format string for arriving messages to current channel from outside the channel.")

(defvar irchat-format-string4 "(%s)"
  "*Format string for arriving messages to other channel from outside the channel.")

(defvar irchat-format-string5 "(%s:%s)"
  "*Format string for arriving messages to other channel from outside the channel.")

(defvar irchat-signoff-msg nil
  "*Default signoff message if no-nil")

(defvar irchat-switching-channels nil
  "*channel list to switch directly*")

(defvar irchat-print-time t
  "*If non-nil, print time prefix on each line in channel buffer*")

(defvar irchat-others-high-watermark 10000
  "*The value of high watermark in Others buffer*")

(defvar irchat-others-low-watermark 5000
  "*The value of high watermark in Others buffer*")

(defvar irchat-beep-on-bells nil
  "*If non-nil, and the IRC Dialogue buffer is not selected in a window,
an IRC message arriving containing a bell character, will cause you
to be notified.
If value is 'always, an arriving bell will always cause a beep (or flash).")

(defvar irchat-ctcp-userinfo "No user information given."
  "*Userinfo message given to anyone asking via CTCP.")

(defvar irchat-timestamp-interval nil
  "*Interval in minutes between timestamps in dialogue-buffer, nil for none.")

(defvar irchat-timestamp-format "*** Time: %s"
  "*Format-string for timestamp.")

(defvar irchat-global-names nil
  "*Set this to T if you want irchat to be able to complete user and 
channel names. If NIL given, irchat does not do global names polling, 
thus causing much less network traffic, but then it is only able to 
complete names on channels you have joined. If number given, it is the
internal between two successive polls.

Notice: There are some gotchas here.
  1. The server sendq is by default 20k and globals names maybe about 30k
     at the time this is written. The NAMES query may cause server to 
     drop the client because of flooding.
  2. The namreply (353) handler is called pretty often and it causes some
     garbage even if it is heavily optimized. You emacs will eat a lot of
     cpu if this is enabled.")

;;
;; additional variables for enhancement by irchat-PJ project
;;

;; added by simm, Sun, Nov 1, 1998
(defvar irchat-pj-filter-level 6
  "Filtering level of Japanese input.
Set this variable integer. Its value means below.

    0 : no process
    1 : enable Emacs20 (same as irchat-2.4jp23b with qh1 and irchat-pj 2.4.23.8 fix)
    2 : add some keybinds
    3 : enable long topic (same as qh3)
    4 : enable DCC chat (same as irchat-pj 2.4.23.11)
    5 : enable SJIS code Japanese
    6 : enable SJIS/EUC-JP code as Japanese (same as qh7)

If you set this variable greater than 6,
irchat-pj behaves this variable as 6.")

(defvar irchat-pj-code-verbose nil
  "Output messages on minibuffer when received non-JIS code")

;; added by simm, Sat, Oct 31, 1998
(defvar irchat-pj-beep-words-alist nil
  "Beep list for irchat-pj.
Set a list of regular expression, and you get beep-sound if match.")

;; added by simm, Mon, Nov 2, 1998
(defvar irchat-pj-beep-when-invited nil
  "Output beep sound when invited.")

(defvar irchat-pj-beep-when-private nil
  "Output beep sound when received private message.")

;; added by simm, Thu, Nov 19, 1998 (pj18)
(defvar irchat-pj-rewrite-server-notice nil
  "Rewrite notice message from server.
Default : =irc.kyoto.wide.ad.jp= gaga
If t    : === gaga")

(defvar irchat-pj-katakana-convert t
  "Convert jisx0201 katakana characters to jisx0208.")

(defvar irchat-pj-fix-gaga-problem t
  "When `gaga' problem occured, not join but warn.")

;;;
;;; eof
;;;
