;;; irchat-pj-modeline.el --- Mode line format definition for irchat-pj

;; Copyright (C) 2000 by Project Pure.

;; Author: SHIMADA Mitsunobu <simm@irc.fan.gr.jp>
;; Keywords: IRC, irchat-pj, mode-line-format

;; $Id: irchat-pj-modeline.el,v 1.4 2000/08/24 16:34:12 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:

(or (boundp 'emacs-major-version)
    (defvar kanji-flag nil)) ;; for original emacs and mule

;;
;; irchat-pj-modeline-Command
;;

(defvar irchat-pj-modeline-Command-default
  (if (featurep 'meadow)
      '("" irchat-away-indicator irchat-freeze-indicator ""
        ((minibuffer-window-selected
	  (display-minibuffer-mode "m" " ") " ")
	 mw32-ime-mode-line-state-indicator)
        " irchat-pj: Commands" irchat-private-indicator
        " <" irchat-servername "> " irchat-nickname
	" " global-mode-string
        " (" irchat-chanbuf-indicator (skk-mode skk-input-mode-string) ")"
        " -%-")
    '("" irchat-away-indicator irchat-freeze-indicator ""
      ((cond ((boundp 'MULE) mc-flag)
	     ((boundp 'kanji-flag) kanji-flag))
       ((minibuffer-window-selected
         (display-minibuffer-mode "m" " ") " ")
        "["
        (minibuffer-window-selected
         (display-minibuffer-mode
          mode-line-egg-mode-in-minibuffer
          mode-line-egg-mode)
         mode-line-egg-mode)
        " ]" mode-line-canna-mode))
      " irchat-pj: Commands" irchat-private-indicator
      " <" irchat-servername "> " irchat-nickname
      " " global-mode-string
      " (" irchat-chanbuf-indicator (skk-mode skk-input-mode-string) ")"
      " -%-")))

(defvar irchat-pj-modeline-Command-for-egg
  '("" irchat-away-indicator irchat-freeze-indicator ""
    ((cond ((boundp 'MULE) mc-flag)
	   ((boundp 'kanji-flag) kanji-flag))
     ((minibuffer-window-selected
       (display-minibuffer-mode "m" " ") " ")
      "["
      (minibuffer-window-selected
       (display-minibuffer-mode
	mode-line-egg-mode-in-minibuffer
	mode-line-egg-mode)
       mode-line-egg-mode)
      " ] " mode-line-canna-mode))
    " irchat-pj: Commands" irchat-private-indicator
    " <" irchat-servername "> " irchat-nickname
    " " global-mode-string
    " (" irchat-chanbuf-indicator ")"
    " -%-"))

(defvar irchat-pj-modeline-Command-for-canna
  '("" irchat-away-indicator irchat-freeze-indicator ""
    ((cond ((boundp 'MULE) mc-flag)
	   ((boundp 'kanji-flag) kanji-flag))
     ((minibuffer-window-selected
       (display-minibuffer-mode "m" " ") " ")
      "-" mode-line-canna-mode))
    " irchat-pj: Commands" irchat-private-indicator
    " <" irchat-servername "> " irchat-nickname
    " " global-mode-string
    " (" irchat-chanbuf-indicator (skk-mode skk-input-mode-string) ")"
    " -%-"))

(defvar irchat-pj-modeline-Command-for-mw32
  '("" irchat-away-indicator irchat-freeze-indicator ""
    ((minibuffer-window-selected
      (display-minibuffer-mode "m" " ") " ")
     mw32-ime-mode-line-state-indicator)
    " irchat-pj: Commands" irchat-private-indicator
    " <" irchat-servername "> " irchat-nickname
    " " global-mode-string
    " (" irchat-chanbuf-indicator  ")"
    " -%-"))

(defvar irchat-pj-modeline-Command-simple
  '("" irchat-away-indicator irchat-freeze-indicator ""
    " irchat-pj: Commands" irchat-private-indicator
    " <" irchat-servername "> " irchat-nickname
    " " global-mode-string
    " (" irchat-chanbuf-indicator (skk-mode skk-input-mode-string) ")"
    " -%-"))

(defvar irchat-pj-modeline-Command-minor-mode
  '("" irchat-away-indicator irchat-freeze-indicator ""
    " irchat-pj: Commands" irchat-private-indicator
    " <" irchat-servername "> " irchat-nickname
    " " global-mode-string
    " (" irchat-chanbuf-indicator minor-mode-alist ")"
    " -%-"))

(defvar irchat-pj-modeline-Command 'irchat-pj-modeline-Command-default
  "Mode line format for irchat-pj Command buffer.

We irchat-PJ project provides below:
\(for most use\)
  irchat-pj-modeline-Command-default
\(for minor-mode kana-kanji converting system, like SKK10\)
  irchat-pj-modeline-Command-minor-mode
\(for non-kana-kanji-conversion, or SKK user\)
  irchat-pj-modeline-Command-simple
\(for Wnn user\)
  irchat-pj-modeline-Command-for-egg
\(for Canna user\)
  irchat-pj-modeline-Command-for-canna
\(for Meadow user with Windoze IM\)
  irchat-pj-modeline-Command-for-mw32
So, choose above, or select your defined mode line.

ex.
\(setq irchat-pj-modeline-Command 'irchat-pj-modeline-Command-minor-mode\)

Default is irchat-pj-modeline-Command-default.
")

;;
;; irchat-pj-modeline-Dialog
;;

(defvar irchat-pj-modeline-Dialog-default
  '("" irchat-away-indicator irchat-freeze-indicator
    " (" irchat-chanbufs-indicator ") "
    (-3 . "%p") " -%-"))

(defvar irchat-pj-modeline-Dialog-minor-mode
  '("" irchat-away-indicator irchat-freeze-indicator
    " (" irchat-chanbufs-indicator minor-mode-alist ") "
    (-3 . "%p") " -%-"))

(defvar irchat-pj-modeline-Dialog 'irchat-pj-modeline-Dialog-default
  "Mode line format for irchat-pj Dialogue buffer.

We irchat-PJ project provides below:
  irchat-pj-modeline-Dialog-default
  irchat-pj-modeline-Dialog-minor-mode
So, choose above, or select your defined mode line.

ex.
\(setq irchat-pj-modeline-Dialog 'irchat-pj-modeline-Dialog-minor-mode\)

Default is irchat-pj-modeline-Dialog-default.
")

;;
;; irchat-pj-modeline-Other
;;

(defvar irchat-pj-modeline-Other-default
  '("" irchat-away-indicator "-"
    " (" irchat-chanbufs-indicator ") "
    (-3 . "%p") " -%-"))

(defvar irchat-pj-modeline-Other-minor-mode
  '("" irchat-away-indicator "-"
    " (" irchat-chanbufs-indicator minor-mode-alist ") "
    (-3 . "%p") " -%-"))

(defvar irchat-pj-modeline-Other 'irchat-pj-modeline-Other-default
  "Mode line format for irchat-pj Other buffer.

We irchat-PJ project provides below:
  irchat-pj-modeline-Other-default
  irchat-pj-modeline-Other-minor-mode
So, choose above, or select your defined mode line.

ex.
\(setq irchat-pj-modeline-Other 'irchat-pj-modeline-Other-minor-mode\)

Default is irchat-pj-modeline-Other-default.
")

;;
;; irchat-pj-modeline-Channel
;;

(defvar irchat-pj-modeline-Channel-default
  '("" irchat-beep-indicator-local irchat-freeze-indicator-local
    irchat-suppress-indicator-local
    "irchat-pj: Current" 
    " (" irchat-chanbuf-indicator ") "
    (-3 . "%p") " -%-"))

(defvar irchat-pj-modeline-Channel-minor-mode
  '("" irchat-beep-indicator-local irchat-freeze-indicator-local
    irchat-suppress-indicator-local
    "irchat-pj: Current" 
    " (" irchat-chanbuf-indicator minor-mode-alist ") "
    (-3 . "%p") " -%-"))

(defvar irchat-pj-modeline-Channel 'irchat-pj-modeline-Channel-default
  "Mode line format for irchat-pj Channel buffer.

We irchat-PJ project provides below:
  irchat-pj-modeline-Channel-default
  irchat-pj-modeline-Channel-alternative
  irchat-pj-modeline-Channel-minor-mode
So, choose above, or select your defined mode line.

ex.
\(setq irchat-pj-modeline-Channel 'irchat-pj-modeline-Channel-minor-mode\)

Default is irchat-pj-modeline-Channel-default.
")

;; That's all
(provide 'irchat-pj-modeline)

;;; irchat-pj-modeline.el ends here
