;;;
;;; $Id: irchat-filter.el,v 1.6 2000/11/04 18:22:39 simm Exp $
;;;
;;; see file irchat-copyright.el for change log and copyright info

(require 'pure-cs)

;;;
;;;  These are defsubst just for speed, as it is expensive to call funtions at
;;;  emacs lisp (also evals are expensive)
;;;
(defsubst irchat-current-minute ()
  (string-to-int (substring (current-time-string) 14 16)))
(defvar irchat-minute 0)

(defun irchat-handle (line)
  "Called when we have at least one line of output from the IRC server."
  (let ((obuf (current-buffer))
	prefix userhost (cmd "") arg)
    (while (string-match "^\\([^\r]*\\)\r+\\(.*\\)" line)
      (setq line (concat (matching-substring line 1)
			 (matching-substring line 2))))
    (if (string-match "^:\\([^ ]+\\) *\\(.*\\)" line)
	(setq prefix (matching-substring line 1)
	      line (matching-substring line 2)))
    (if (and prefix (string-match "^\\([^!]*\\)!\\(.*\\)" prefix))
	(setq userhost (matching-substring prefix 2)
	      prefix (matching-substring prefix 1)))
    (if (string-match "^ *\\([^ ]+\\) *\\(.*\\)" line)
	(setq arg (list (matching-substring line 1))
	      line (matching-substring line 2)))
    (while (string-match "^\\([^:][^ ]*\\) +\\(.*\\)" line)
      (nconc arg (list (matching-substring line 1)))
      (setq line (matching-substring line 2)))
    (if (not (string= line ""))
	(if (string-match "^:\\(.*\\)" line)
	    (nconc arg (list (matching-substring line 1)))
	  (nconc arg (list line))))
    (if (and prefix userhost
	     (not (string= (get (intern prefix) 'userhost) userhost)))
	(put (intern prefix) 'userhost userhost))
    (if (car arg)
	(setq cmd (downcase (car arg))
	      arg (cdr arg)))
    (if (and (boundp (setq hook
			   (intern (concat "irchat-handle-" cmd "-hook"))))
	     (eval hook)
	     (eval (list (eval hook) (cons prefix arg))))
	nil
      (setq fun (intern (concat "irchat-handle-" cmd)))
      ;; modified by simm@irc.fan.gr.jp, Thu, 10 Jun 1999
      (if (or (eq fun 'irchat-handle-join)
              (eq fun 'irchat-handle-nick)
              (eq fun 'irchat-handle-part)
              (eq fun 'irchat-handle-quit))
          (apply fun prefix userhost arg)
        (if (fboundp fun)
            (apply fun prefix arg)
          (if irchat-debugging
              (irchat-insert-special (format "No handle-%s (%s) %s\n" cmd prefix
                                             (prin1-to-string arg))))
          (let* ((cmd-number (string-to-int cmd))
                 (default-number (/ cmd-number 100)) tmp)
            (setq fun (intern (concat "irchat-handle-"
                                      (format "%d00s" default-number))))
            (if (and (> cmd-number 0)
                     (fboundp fun))
                (apply fun cmd-number prefix arg)
              (setq tmp (format "Unknown MESSAGE: [%s] [%s] [%s]\n" 
                                prefix cmd (prin1-to-string arg)))
              (message tmp)
              (irchat-insert0 tmp))))))
    (set-buffer obuf)))

(defun irchat-pj-filter (proc string)
  "Filter function for irchat-pj server process."
  (let ((md (match-data)))
    (save-excursion
      (set-buffer irchat-pj-debug-buffer)
      (goto-char (point-max))
      (insert string)
      (set-buffer (process-buffer proc))
      (goto-char (point-max))
      (insert string)
      (while (and (goto-char (point-min))
		  (= 0 (forward-line 1))
		  (bolp))
	(save-restriction
	  (narrow-to-region (point-min) (point))
	  (goto-char (point-max))
	  (while (or (= 10 (preceding-char)) (= 13 (preceding-char)))
	    (delete-backward-char 1))
	  (if irchat-pj-cs-detect-function
	      (let ((cs (funcall irchat-pj-cs-detect-function (point-min) (point-max))))
		(or (and (eq cs 'euc-jp)
			 irchat-pj-cs-verbose
			 (message "EUC-JP code detected."))
		    (and (eq cs 'shift_jis)
			 irchat-pj-cs-verbose
			 (message "SHIFT JIS code detected.")))
		(irchat-handle
		 (pure-cs-decode-string
		  (buffer-substring (point-min) (point-max)) cs))))
	  (delete-region (point-min) (point-max))))
      (if (/= (irchat-current-minute) irchat-minute)
	  (let (fun)
	    (setq irchat-minute (irchat-current-minute))
	    (if (fboundp (setq fun (intern "irchat-minutely")))
		(apply fun nil))))
      (store-match-data md))))

(defun irchat-sentinel (proc status)
  "Sentinel function for IRC server process."
  (and irchat-server-process
       (not (irchat-server-opened))
       (cond ((or irchat-reconnect-automagic irchat-reconnect-with-password)
	      (or (and irchat-grow-tail
		       (not irchat-reconnect-with-password)
		       (setq irchat-nickname (concat irchat-nickname irchat-grow-tail))
		       (irchat 'always))
		  (irchat)))
	     (irchat-fatal-error-message
	      (error (format "IRC ERROR: %s" irchat-fatal-error-message)))
	     ((process-id proc)
	      (irchat-sentinel2 proc status))
	     (t
	      (error (format "IRC ERROR: Connection closed. (%s)"
			     (substring status 0 (1- (length status)))))))))

(defun irchat-sentinel2 (proc status)
  (if (string-match "^exited abnormally with code \\([0-9]+\\)" status)
      (let ((status (string-to-int (matching-substring status 1))))
	(cond
	 ((= 99 status);; unsupported command
	  (error (format "IRC ERROR: Please use a newer \"%s\"."
			 irchat-pj-dcc-program)))
	 ((= 98 status);; bad argment number
	  (error (format "IRC ERROR: Please use a newer \"%s\"."
			 irchat-pj-dcc-program)))
	 ((= 97 status)
	  (error (format "IRC ERROR: Cannot connect to IRC server.")))
	 (t
	  (error (format "IRC ERROR: Server connection closed.")))))
    (error (format "IRC ERROR: Connection closed. (%s)"
		   (substring status 0 (1- (length status)))))))

;; That's all
(provide 'irchat-filter)

;;;
;;; eof
;;;
