;;; irchat-pj-dcc.el --- interface for dcc process

;; Copyright (C) 2000 by Project Pure

;; Author: SHIMADA Mitsunobu <simm@irc.fan.gr.jp>
;; Keywords: IRC, irchat-pj, PURE, DCC

;; $Id: irchat-pj-dcc.el,v 1.3 2000/11/06 14:31:24 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:

(require 'pure-cs)
(require 'pure-irc-send)
(require 'pure-irc-dcc)

;; check resume enable

(defun irchat-pj-dcc-check-resume-ok ()
  "Check dcc process to support DCC RESUME feature."
  (interactive)
  (let (result path rest)
    (if (file-executable-p irchat-pj-dcc-program)
	(setq result irchat-pj-dcc-program)
      (setq rest exec-path)
      (while rest
	(setq path (expand-file-name irchat-pj-dcc-program (car rest))
	      rest (cdr rest))
	(if (file-executable-p path)
	    (setq rest nil result path))))
    (setq pure-irc-dcc-support-resume
	  (and result
	       (= 0 (call-process irchat-pj-dcc-program nil nil nil "check" "resume"))))))

;; DCC server process execute

(defun irchat-pj-dcc-start-client (dinfo &optional parent coding resume)
  "Start DCC GET process."
  (if (not (member dinfo pure-irc-dcc-offer-list))
      (funcall pure-irc-dcc-error-function "not offered")
    (let ((host (pure-irc-dcc-info-get-host dinfo))
	  (port (pure-irc-dcc-info-get-port dinfo))
	  (file (pure-irc-dcc-info-get-file dinfo))
	  (size (pure-irc-dcc-info-get-size dinfo))
	  proc buf)
      (setq buf (generate-new-buffer (concat " pure:DCC:client"))
	    proc (cond
		  ((and size resume pure-irc-dcc-support-resume)
		   (start-process irchat-pj-dcc-program buf
				  irchat-pj-dcc-program "file" "rget" host port size file))
		  (size
		   (start-process irchat-pj-dcc-program buf
				  irchat-pj-dcc-program "file" "get" host port size file))
		  (t
		   (start-process
		    (or irchat-pj-dcc-chat-client-program irchat-pj-dcc-program) buf
		    irchat-pj-dcc-program "chat" "connect" host port))))
      (if size
	  (set-process-sentinel proc 'irchat-pj-dcc-sentinel)
	(set-process-sentinel proc 'pure-irc-dcc-chat-sentinel))
      (pure-irc-dcc-offer-remove dinfo)
      (irchat-pj-dcc-start-common proc buf 'connect parent coding dinfo))))

(defun irchat-pj-dcc-start-server (dinfo &optional parent coding)
  "Start DCC server process.
If DCC SEND, file is filename string.
If DCC CHAT, file is nil."
  (let ((nick (pure-irc-dcc-info-get-nick dinfo))
	(host (pure-irc-dcc-info-get-host dinfo))
	(port (pure-irc-dcc-info-get-port dinfo))
	(file (pure-irc-dcc-info-get-file dinfo))
	buf proc kind1 kind2)
    (pure-irc-dcc-info-put-current-time dinfo)
    (setq buf (generate-new-buffer (concat " pure:DCC:server"))
	  proc (cond ((and file host)
		      (start-process irchat-pj-dcc-program buf
				     irchat-pj-dcc-program "file" "send" file port host))
		     ((and file port)
		      (start-process irchat-pj-dcc-program buf
				     irchat-pj-dcc-program "file" "send" file port))
		     (file
		      (start-process irchat-pj-dcc-program buf
				     irchat-pj-dcc-program "file" "send" file))
		     (host
		      (start-process irchat-pj-dcc-program buf
				     irchat-pj-dcc-program "chat" "listen" port host))
		     (port
		      (start-process irchat-pj-dcc-program buf
				     irchat-pj-dcc-program "chat" "listen" port))
		     (t
		      (start-process irchat-pj-dcc-program buf
				     irchat-pj-dcc-program "chat" "listen"))))
    (if file
	(set-process-sentinel proc 'irchat-pj-dcc-sentinel)
      (set-process-sentinel proc 'pure-irc-dcc-chat-sentinel))
    (irchat-pj-dcc-start-common proc buf 'set parent coding dinfo)))

(defun irchat-pj-dcc-start-common (proc buf stat parent coding dinfo)
  "Common behaviour when both DCC server and client process start."
  (setq pure-irc-dcc-process-list (cons proc pure-irc-dcc-process-list))
  (pure-cs-set-noconv-process proc)
  (set-process-filter proc 'irchat-pj-dcc-filter)
  (save-excursion
    (set-buffer buf)
    (pure-bl-make-current pure-irc-dcc-local-variables t)
    (setq pure-irc-dcc-status stat
	  pure-irc-dcc-parent parent
	  pure-irc-dcc-coding coding
	  pure-irc-dcc-info   dinfo))
  proc)

;; filter, sentinel for DCC process
(defun irchat-pj-dcc-filter (proc msg)
  "DCC filter function."
  (let (beg end)
    (save-excursion
      (set-buffer (process-buffer proc))
      (goto-char (point-max))
      (insert msg)
      (while (and (goto-char (point-min))
		  (= 0 (forward-line 1))
		  (bolp))
	(if (= 10 (char-before)) (delete-backward-char 1))
	(if (= 13 (char-before)) (delete-backward-char 1))
	(setq end (point) beg (point-min))
	(goto-char (point-min))
	(cond ((looking-at "DCC SEND \\([^ ]+\\) \\([^ ]+\\) \\([^ ]+\\)")
	       (pure-irc-dcc-send-set-params
		(buffer-substring (match-beginning 1) (match-end 1))
		(buffer-substring (match-beginning 2) (match-end 2))
		(buffer-substring (match-beginning 3) (match-end 3))))
	      ((looking-at "DCC CHAT \\([^ ]+\\) \\([^ ]+\\)")
	       (pure-irc-dcc-chat-set-params
		(buffer-substring (match-beginning 1) (match-end 1))
		(buffer-substring (match-beginning 2) (match-end 2))))
	      ((looking-at "DCC RESUME \\([^ ]+\\) \\([^ ]+\\) \\([^ ]+\\)")
	       (if (not pure-irc-dcc-support-resume)
		   (message "DCC WARNING: not support DCC RESUME")
		 (pure-irc-dcc-resume-request
		  (buffer-substring (match-beginning 3) (match-end 3)))))
	      ((looking-at "DCC ACCEPT \\([^ ]+\\) \\([^ ]+\\) \\([^ ]+\\)")
	       (if (not pure-irc-dcc-support-resume)
		   (message "DCC WARNING: not support DCC RESUME")
		 (pure-irc-dcc-resume-accept
		  (buffer-substring (match-beginning 3) (match-end 3)))))
	      ((looking-at "DCC CHATTING")
	       (let ((nick (pure-irc-dcc-info-get-nick)))
		 (or (assoc nick pure-irc-dcc-chat-partner)
		     (setq pure-irc-dcc-chat-partner
			   (cons (cons nick proc) pure-irc-dcc-chat-partner)))
		 (set-process-filter proc 'pure-irc-dcc-chat-filter)
		 (funcall pure-irc-dcc-log-interface
			  "DCC CHAT to %s established." nick))
	       (setq pure-irc-dcc-status
		     (if (eq pure-irc-dcc-status 'connect) 'client 'server)))
	      ((looking-at "DCC SENDING")
	       (funcall pure-irc-dcc-log-interface
			"DCC SEND to %s established (%s, %s bytes)."
			(pure-irc-dcc-info-get-nick)
			(pure-irc-dcc-info-get-file)
			(pure-irc-dcc-info-get-size))
	       (setq pure-irc-dcc-status 'server))
	      ((looking-at "DCC GETTING")
	       (funcall pure-irc-dcc-log-interface
			"DCC GET from %s established (%s, %s bytes)."
			(pure-irc-dcc-info-get-nick)
			(pure-irc-dcc-info-get-file)
			(pure-irc-dcc-info-get-size))
	       (setq pure-irc-dcc-status 'client))
	      ((looking-at "DCC REPORT ")
	       (message "DCC REPORT: %s" (buffer-substring (match-end 0) end)))
	      ((looking-at "DCC WARNING Not Support Send Resume")
	       (message "DCC WARNING: not support send resume")
	       (setq pure-irc-dcc-support-resume nil))
	      ((looking-at "DCC WARNINIG ")
	       (message "DCC WARNINIG: %s" (buffer-substring (match-end 0) end)))
	      ((looking-at "DCC WARN ")
	       (message "DCC WARNINIG: %s" (buffer-substring (match-end 0) end)))
	      ((looking-at "DCC ERROR1 ")
	       (funcall pure-irc-dcc-log-interface
			"DCC ERROR: %s" (buffer-substring (match-end 0) end))
	       (pure-irc-dcc-offer-add pure-irc-dcc-info))
	      ((looking-at "DCC ERROR ")
	       (funcall pure-irc-dcc-log-interface
			"DCC ERROR: %s" (buffer-substring (match-end 0) end)))
	      (t ; unknown
	       (funcall pure-irc-dcc-log-interface
			"DCC FATAL ERROR: %s" (buffer-substring beg end))))
	(delete-region beg end)))))

(defun irchat-pj-dcc-sentinel (proc msg)
  "DCC sentinel function."
  (let ((stat (process-status proc))
	(buf  (process-buffer proc)))
    (if (or (eq stat 'run) (eq stat 'stop)) nil
      (save-excursion
	(set-buffer buf)
	(if (string-match "^finished" msg)
	    (if (eq 'server pure-irc-dcc-status)
		(funcall pure-irc-dcc-log-interface "DCC finished to send %s to %s"
			 (pure-irc-dcc-info-get-file) (pure-irc-dcc-info-get-nick))
	      ;; (eq 'client pure-irc-dcc-status)
	      (funcall pure-irc-dcc-log-interface "DCC finished to get %s from %s"
		       (pure-irc-dcc-info-get-file) (pure-irc-dcc-info-get-nick)))
	  (if (string-match "[\r\n]+$" msg)
	      (setq msg (substring msg 0 (match-beginning 0))))
	  (funcall pure-irc-dcc-log-interface "DCC ERROR and stop DCC %s with %s: %s"
		   (pure-irc-dcc-info-get-file) (pure-irc-dcc-info-get-nick) msg))
	(if (pure-irc-dcc-is-chat)
	    (setq pure-irc-dcc-chat-partner
		  (delete (assoc (pure-irc-dcc-info-get-nick) pure-irc-dcc-chat-partner)
			  pure-irc-dcc-chat-partner))))
      (setq pure-irc-dcc-process-list (delete proc pure-irc-dcc-process-list))
      (set-process-buffer proc nil)
      (set-process-filter proc nil)
      (set-process-sentinel proc nil)
      (kill-buffer buf))))

;; That's all
(provide 'irchat-pj-dcc)

;;; pure-irc-dcc.el ends here
