;;; pure-cs-kana.el --- Converter module JISX0201 -> JISX0208 for PURE coding-system module

;; Copyright (C) 2000 by Project Pure.

;; Author: SHIMADA Mitsunobu <simm@irc.fan.gr.jp>
;; Keywords: PURE, coding-system, ISO-2022, JISX0201, $(BH>3Q%+%J(B

;; $Id: pure-cs-kana.el,v 1.1.1.1 2000/07/31 18:57:39 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; PURE means "Primitive Universal Relay-chat Environment"
;; CS means coding-system
;; This module is derived from irchat-pj-jisx0201.el

;;; Code:

(defconst pure-cs-kana-twobytes-character-alist
  '(("(I6^(B" . "$(B%,(B") ("(I7^(B" . "$(B%.(B") ("(I8^(B" . "$(B%0(B") ("(I9^(B" . "$(B%2(B") ("(I:^(B" . "$(B%4(B")
    ("(I;^(B" . "$(B%6(B") ("(I<^(B" . "$(B%8(B") ("(I=^(B" . "$(B%:(B") ("(I>^(B" . "$(B%<(B") ("(I?^(B" . "$(B%>(B")
    ("(I@^(B" . "$(B%@(B") ("(IA^(B" . "$(B%B(B") ("(IB^(B" . "$(B%E(B") ("(IC^(B" . "$(B%G(B") ("(ID^(B" . "$(B%I(B")
    ("(IJ^(B" . "$(B%P(B") ("(IK^(B" . "$(B%S(B") ("(IL^(B" . "$(B%V(B") ("(IM^(B" . "$(B%Y(B") ("(IN^(B" . "$(B%\(B")
    ("(IJ_(B" . "$(B%Q(B") ("(IK_(B" . "$(B%T(B") ("(IL_(B" . "$(B%W(B") ("(IM_(B" . "$(B%Z(B") ("(IN_(B" . "$(B%](B")
    ("(I3^(B" . "$(B%t(B")))

(defconst pure-cs-kana-onebyte-character-alist
  '(("(I'(B" . "$(B%!(B") ("(I((B" . "$(B%#(B") ("(I)(B" . "$(B%%(B") ("(I*(B" . "$(B%'(B") ("(I+(B" . "$(B%)(B")
    ("(I,(B" . "$(B%c(B") ("(I-(B" . "$(B%e(B") ("(I.(B" . "$(B%g(B") ("(I/(B" . "$(B%C(B")
    ("(I1(B" . "$(B%"(B") ("(I2(B" . "$(B%$(B") ("(I3(B" . "$(B%&(B") ("(I4(B" . "$(B%((B") ("(I5(B" . "$(B%*(B")
    ("(I6(B" . "$(B%+(B") ("(I7(B" . "$(B%-(B") ("(I8(B" . "$(B%/(B") ("(I9(B" . "$(B%1(B") ("(I:(B" . "$(B%3(B")
    ("(I;(B" . "$(B%5(B") ("(I<(B" . "$(B%7(B") ("(I=(B" . "$(B%9(B") ("(I>(B" . "$(B%;(B") ("(I?(B" . "$(B%=(B")
    ("(I@(B" . "$(B%?(B") ("(IA(B" . "$(B%A(B") ("(IB(B" . "$(B%D(B") ("(IC(B" . "$(B%F(B") ("(ID(B" . "$(B%H(B")
    ("(IE(B" . "$(B%J(B") ("(IF(B" . "$(B%K(B") ("(IG(B" . "$(B%L(B") ("(IH(B" . "$(B%M(B") ("(II(B" . "$(B%N(B")
    ("(IJ(B" . "$(B%O(B") ("(IK(B" . "$(B%R(B") ("(IL(B" . "$(B%U(B") ("(IM(B" . "$(B%X(B") ("(IN(B" . "$(B%[(B")
    ("(IO(B" . "$(B%^(B") ("(IP(B" . "$(B%_(B") ("(IQ(B" . "$(B%`(B") ("(IR(B" . "$(B%a(B") ("(IS(B" . "$(B%b(B")
    ("(IT(B" . "$(B%d(B") ("(IU(B" . "$(B%f(B") ("(IV(B" . "$(B%h(B")
    ("(IW(B" . "$(B%i(B") ("(IX(B" . "$(B%j(B") ("(IY(B" . "$(B%k(B") ("(IZ(B" . "$(B%l(B") ("(I[(B" . "$(B%m(B")
    ("(I\(B" . "$(B%o(B") ("(I&(B" . "$(B%r(B") ("(I](B" . "$(B%s(B")
    ("(I$(B" . "$(B!"(B") ("(I!(B" . "$(B!#(B") ("(I%(B" . "$(B!&(B")
    ("(I^(B" . "$(B!+(B") ("(I_(B" . "$(B!,(B") ("(I0(B" . "$(B!<(B") ("(I"(B" . "$(B!V(B") ("(I#(B" . "$(B!W(B")))

(defun pure-cs-kana-convert-region (beg end)
  "Convert JISX0201 kana code into JISX0208"
  (interactive "r")
  (let (ch)
    (save-excursion
      (goto-char beg)
      (save-restriction
	(while (re-search-forward "\\ck" end t)
	  (setq ch (char-before))
	  (cond ((and (= ch ?(I%(B) (looking-at "(I%(B"))
		 (delete-backward-char 1)
		 (if (looking-at "(I%%(B")
		     (progn
		       (delete-char 2)
		       (insert "$(B!D(B"))
		   (delete-char 1)
		   (insert "$(B!E(B")))
		((looking-at "[(I^_(B]")
		 (backward-char)
		 (setq ch (cdr (assoc (buffer-substring (point) (+ 2 (point)))
				      pure-cs-kana-twobytes-character-alist)))
		 (if ch
		     (progn
		       (delete-char 2)
		       (insert ch))
		   (setq ch (cdr (assoc (buffer-substring (point) (1+ (point)))
					pure-cs-kana-onebyte-character-alist)))
		   (insert (or ch " "))))
		(t
		 (setq ch (cdr (assoc (buffer-substring (1- (point)) (point))
				      pure-cs-kana-onebyte-character-alist)))
		 (delete-backward-char 1)
		 (insert (or ch " ")))))))))
	



;;; That's all
(provide 'pure-cs-kana)

;;; pure-cs-kana.el ends here
