;;; pure-cs-xmas.el --- XEmacs specific code for PURE coding-system module

;; Copyright (C) 2000 by Project Pure.

;; Author: SHIMADA Mitsunobu <simm@irc.fan.gr.jp>
;; Keywords: PURE, coding-system, XEmacs21.*

;; $Id: pure-cs-xmas.el,v 1.1.1.1 2000/07/31 18:57:39 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; PURE means "Primitive Universal Relay-chat Environment"
;; CS means coding-system
;; This module is derived from irchat-pj-specific-xmas.el

;;; Code:

(defvar pure-cs-enable-mbchars (featurep 'mule)
  "Multibyte characters availability flag.
If non-nil, multibyte characters are available.")

(fset 'pure-cs-insert 'insert)

(if pure-cs-enable-mbchars
    (make-coding-system
     'iso-2022-jproxy 'iso2022
     "ISO-2022 based 8bit encoding using SS2 as JISX0201"
     '(mnemonic "ISO8/JProxy"
		charset-g0 ascii
		charset-g1 katakana-jisx0201
		charset-g2 t)))

(defsubst pure-cs-set-service (service in out)
  (modify-coding-system-alist 'network service (cons in out)))
(defsubst pure-cs-set-process (process in out)
  (set-process-coding-system process in out))

(defsubst pure-cs-set-noconv-service (service)
  (pure-cs-set-service service 'no-conversion 'no-conversion))
(defsubst pure-cs-set-noconv-process (process)
  (pure-cs-set-process process 'no-conversion 'no-conversion))

(fset 'pure-cs-decode-region 'decode-coding-region)
(fset 'pure-cs-encode-region 'encode-coding-region)
(fset 'pure-cs-decode-string 'decode-coding-string)
(fset 'pure-cs-encode-string 'encode-coding-string)

;;; That's all
(provide 'pure-cs-xmas)

;;; pure-cs-xmas.el ends here
