;;; pure-vs-e20.el --- GNU Emacs20.* specific code for PURE version-string module

;; Copyright (C) 2000 by Project Pure.

;; Author: SHIMADA Mitsunobu <simm@irc.fan.gr.jp>
;; Keywords: PURE, version-string, Emacs20.*

;; $Id: pure-vs-e20.el,v 1.1.1.1 2000/07/31 18:57:39 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; PURE means "Primitive Universal Relay-chat Environment."
;; VS means version-string
;; This module is derived from irchat-pj-specific-emacs20.el

;;; Code:

(defconst pure-vs-emacs
  (let ((vs-emacs
	 (cond ((string-match "^\\([0-9]+\\.[0-9]+\\.[0-9]+\\)\\.[0-9]+" emacs-version)
		(substring emacs-version (match-beginning 1) (match-end 1)))
	       ((string-match "^\\([0-9]+\\.[0-9]+\\)\\.[0-9]+" emacs-version)
		(substring emacs-version (match-beginning 1) (match-end 1)))
	       (t emacs-version)))
	(vs-mule
	 (or (and (boundp 'enable-multibyte-characters)
		  enable-multibyte-characters
		  mule-version)
	      nil))
	(vs-meadow
	 (if (featurep 'meadow)
	     (let ((meadow-version (Meadow-version)))
	       (if (string-match "^Meadow-" meadow-version)
		   (substring meadow-version (match-end 0))
		 meadow-version))
	   nil)))
    (cond ((and vs-meadow vs-mule)
	   (format "Meadow/%s (%s) [Emacs/%s,Mule/%s]"
		   vs-meadow system-configuration vs-emacs vs-mule))
	  (vs-meadow
	   (format "Meadow/%s (%s) [Emacs/%s]"
		   vs-meadow system-configuration vs-emacs))
	  (vs-mule
	   (format "Emacs/%s (%s) Mule/%s" vs-emacs system-configuration vs-mule))
	  (t (concat "Emacs/" vs-emacs))))
  "Emacs version string for PURE")

;; That's all
(provide 'pure-vs-e20)

;;; pure-vs-e20.el ends here
