;;; pure-vs-xmas.el --- XEmacs specific code for PURE version-string module

;; Copyright (C) 2000 by Project Pure.

;; Author: Mitsunobu Shimada <shimada@itl.tnr.sharp.co.jp>
;; Keywords: PURE, version-string, XEmacs

;; $Id: pure-vs-xmas.el,v 1.2 2000/10/18 14:46:49 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; PURE means "Primitive Universal Relay-chat Environment."
;; VS means version-string
;; This module is derived from irchat-pj-specific-xmas.el

;;; Code:

(defconst pure-vs-string-emacs
  (let ((emacsver
	 (if (string-match " " emacs-version)
	     (substring emacs-version 0 (match-beginning 0))
	   emacs-version)))
    (cond ((stringp xemacs-betaname)
	   (format "XEmacs %s-%s" emacsver
		   (substring xemacs-betaname 1 (1- (length xemacs-betaname)))))
	  ((numberp emacs-patch-level)
	   (format "XEmacs %s-patch%d" emacsver emacs-patch-level))
	  (t
	   (format "XEMacs %s" emacsver))))
  "Version string of PURE current version")

(defconst pure-vs-emacs
  (let* ((vs-emacs
	  (if (string-match " " emacs-version)
	      (substring emacs-version 0 (match-beginning 0))
	    emacs-version))
	 (vs-beta
	  (and (stringp xemacs-betaname)
	       (concat vs-emacs "-"
		       (substring xemacs-betaname 1 (1- (length xemacs-betaname))))))
	 (vs-patch
	  (and (numberp emacs-patch-level)
	       (format "%s-patch%d" vs-emacs emacs-patch-level)))
	 (vs-mule (featurep 'mule))
	 (vs-utf2000
	  (and (featurep 'utf-2000)
	       (or (and (boundp 'utf-2000-version)
			(string-match " " utf-2000-version)
			(substring utf-2000-version 0 (match-beginning 0)))
		   utf-2000-version)))
	 (vs-addition
	  (if (boundp 'xemacs-codename)
	      (format "\"%s\" (%s)" xemacs-codename system-configuration)
	    (concat "(" system-configuration ")"))))
    (cond (vs-utf2000
	   (format "UTF-200-Mule/%s (%s) [XEmacs/%s \"%s\"]"
		   vs-utf2000
		   system-configuration
		   (or vs-beta vs-patch vs-emacs)
		   xemacs-codename))
	  (vs-mule
	   (format "XEmacs-MULE/%s %s" (or vs-beta vs-patch vs-emacs) vs-addition))
	  (t (format "XEmacs/%s %s" (or vs-beta vs-patch vs-emacs) vs-addition))))
  "Emacs version string for PURE")

;; That's all
(provide 'pure-vs-xmas)

;;; pure-vs-xmas.el ends here
