;;; pure-cs-e20.el --- GNU Emacs20.* specific code for PURE coding-system module

;; Copyright (C) 2000 by Project Pure.

;; Author: SHIMADA Mitsunobu <simm@irc.fan.gr.jp>
;; Keywords: PURE, coding-system, Emacs20.*

;; $Id: pure-cs-e20.el,v 1.3 2000/11/03 14:39:32 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; PURE means "Primitive Universal Relay-chat Environment"
;; CS means coding-system
;; This module is derived from irchat-pj-specific-emacs20.el

;;; Code:

(defvar pure-cs-enable-mbchars enable-multibyte-characters
  "Multibyte characters availability flag.
If non-nil, multibyte characters are available.")

(if pure-cs-enable-mbchars
    (make-coding-system
     'iso-2022-jproxy 2 ?I
     "ISO-2022 based 8bit encoding using SS2 as JISX0201"
     '((ascii t) (katakana-jisx0201 t) t nil
       nil ascii-eol ascii-cntl seven nil single-shift)
     '((safe-charsets . t))))

(defsubst pure-cs-set-service (service in out)
  (modify-coding-system-alist 'network service (cons in out)))
(defsubst pure-cs-set-process (process in out)
  (set-process-coding-system process in out))

(defsubst pure-cs-set-noconv-service (service)
  (pure-cs-set-service service 'no-conversion 'no-conversion))
(defsubst pure-cs-set-noconv-process (process)
  (pure-cs-set-process process 'no-conversion 'no-conversion))

(fset 'pure-cs-decode-region 'decode-coding-region)
(fset 'pure-cs-encode-region 'encode-coding-region)
(fset 'pure-cs-decode-string 'decode-coding-string)
(fset 'pure-cs-encode-string 'encode-coding-string)

(defun pure-cs-buffer-unibyte (&optional buf)
  "Set buffer for unibyte character."
  (save-excursion
    (if buf (set-buffer buf))
    (set-buffer-multibyte nil)))

(defun pure-cs-buffer-multibyte (&optional buf)
  "Set buffer for multibyte character."
  (save-excursion
    (if buf (set-buffer buf))
    (set-buffer-multibyte t)))

(defun pure-cs-detect-jp-region-e20 (beg end)
  "Detect coding system inner region, assumes that given region is written in Japanese."
  (interactive "r")
  (unwind-protect
      (progn
	(pure-cs-buffer-unibyte)
	(pure-cs-detect-jp-region-common beg end))
    (pure-cs-buffer-multibyte)))

;;; That's all
(provide 'pure-cs-e20)

;;; pure-cs-e20.el ends here
