;;; pure-cs.el --- Generic function and loader for PURE coding-system module

;; Copyright (C) 2000 by Project Pure.

;; Author: SHIMADA Mitsunobu <simm@irc.fan.gr.jp>
;; Keywords: PURE, coding-system

;; $Id: pure-cs.el,v 1.3 2000/11/03 14:39:32 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; PURE means "Primitive Universal Relay-chat Environment"
;; CS means coding-system

;;; Code:

(defvar pure-cs-default-encode 'ctext
  "Default encode coding-system")

(cond ((featurep 'xemacs)
       (require 'pure-cs-xmas))
      ((and (boundp 'emacs-major-version) (<= 20 emacs-major-version))
       (require 'pure-cs-e20))
      ((featurep 'mule)
       (require 'pure-cs-mule2))
      ((boundp 'NEMACS)
       (require 'pure-cs-nemacs))
      (t (require 'pure-cs-se)))
(require 'pure-cs-detect)
(if pure-cs-enable-mbchars
    (require 'pure-cs-kana))

(or (fboundp 'pure-cs-buffer-unibyte)
    (defun pure-cs-buffer-unibyte (&optional buf)
      "Set buffer for unibyte character."
      nil))

(or (fboundp 'pure-cs-buffer-multibyte)
    (defun pure-cs-buffer-multibyte (&optional buf)
      "Set buffer for multibyte character."
      t))

(if (fboundp 'pure-cs-detect-jp-region-e20)
    (fset 'pure-cs-detect-jp-region 'pure-cs-detect-jp-region-e20)
  (fset 'pure-cs-detect-jp-region 'pure-cs-detect-jp-region-common))

(or (fboundp 'pure-cs-detect-region)
    (fset 'pure-cs-detect-region 'pure-cs-detect-jp-region))

;; That's all
(provide 'pure-cs)

;;; pure-cs.el ends here
