;;; pure-irc-dcc-file.el --- DCC file management for PURE (with RESUME feature)

;; Copyright (C) 2000 by Project Pure.

;; Author: SHIMADA Mitsunobu <simm@pure.fan.gr.jp>
;; Keywords: 

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; PURE means "Primitive Universal Relay-chat Environment"

;;; Code:

(require 'pure-irc-dcc-common)
(require 'pure-irc-send)

;; set DCC SEND information
(defun pure-irc-dcc-send-set-params (host port size)
  "DCC filter function: get \"DCC SEND \""
  (let ((file (file-name-nondirectory (pure-irc-dcc-info-get-file)))
	(nick (pure-irc-dcc-info-get-nick)))
    (setq pure-irc-dcc-status 'wait)
    (pure-irc-dcc-info-put-host host)
    (pure-irc-dcc-info-put-port port)
    (pure-irc-dcc-info-put-size size)
    (pure-irc-send-register pure-irc-dcc-parent pure-irc-dcc-coding
     "PRIVMSG %s :\001DCC SEND %s %s %s %s\001\n" nick file host port size)
    (funcall pure-irc-dcc-log-interface "DCC SEND to %s: %s (%s bytes)" nick file size)))

;; DCC RESUME / ACCEPT
(defun pure-irc-dcc-resume-find-process (nick port)
  "Find DCC wait process, `nick' and `port' corresponds."
  (let (proc (rest pure-irc-dcc-process-list))
    (save-excursion
      (while rest
	(setq proc (car rest))
	(set-buffer (process-buffer proc))
	(or (and (or (eq pure-irc-dcc-status 'wait)
		     (eq pure-irc-dcc-status 'resume))
		 (string= nick (pure-irc-dcc-info-get-nick))
		 (string= port (pure-irc-dcc-info-get-port))
		 (setq rest nil proc proc))
	    (setq proc nil rest (cdr rest))))
      proc)))

(defun pure-irc-dcc-resume-request (pos &optional proc)
  "Send DCC RESUME message to peer."
  (save-excursion
    (if (processp proc)
	(set-buffer (process-buffer proc)))
    (let ((nick (pure-irc-dcc-info-get-nick))
	  (file (pure-irc-dcc-info-get-file))
	  (port (pure-irc-dcc-info-get-port)))
      (setq pure-irc-dcc-status 'resume)
      (pure-irc-send-register pure-irc-dcc-parent pure-irc-dcc-coding
       "PRIVMSG %s :\001DCC RESUME %s %s %s\001\n" nick (file-name-nondirectory file) port pos)
      (funcall pure-irc-dcc-log-interface
	       "DCC RESUME request to %s: %s (at %s bytes)" nick file pos))))

(defun pure-irc-dcc-resume-accept (pos &optional proc)
  "Send DCC ACCEPT message to peer."
  (save-excursion
    (if (processp proc)
	(set-buffer (process-buffer proc)))
    (let ((nick (pure-irc-dcc-info-get-nick))
	  (file (pure-irc-dcc-info-get-file))
	  (port (pure-irc-dcc-info-get-port)))
      (pure-irc-send-register pure-irc-dcc-parent pure-irc-dcc-coding
       "PRIVMSG %s :\001DCC ACCEPT %s %s %s\001\n" nick (file-name-nondirectory file) port pos)
      (funcall pure-irc-dcc-log-interface
	       "DCC RESUME accepted from %s: %s (at %s bytes)" nick file pos))))

(defun pure-irc-dcc-resume-cancel (nick &optional port)
  "Cancel DCC RESUME and send request back to `pure-irc-dcc-offer-list'."
  (let (proc (rest pure-irc-dcc-process-list))
    (save-excursion
      (while rest
	(setq proc (car rest)
	      rest (cdr rest))
	(set-buffer (process-buffer proc))
	(and (eq pure-irc-dcc-status 'resume)
	     (string= nick (pure-irc-dcc-info-get-nick))
	     (or (and port (string= port (pure-irc-dcc-info-get-port))) t)
	     (progn
	       (pure-irc-dcc-info-put-file
		(file-name-nondirectory (pure-irc-dcc-info-get-file)))
	       (setq pure-irc-dcc-process-list (delete proc pure-irc-dcc-process-list)
		     pure-irc-dcc-offer-list (cons pure-irc-dcc-info pure-irc-dcc-offer-list))
	       (funcall pure-irc-dcc-log-interface
			"DCC RESUME \"%s\" disabled, so send it back to offer list."
			(pure-irc-dcc-info-get-file))
	       (delete-process proc)))))))

;; That's all
(provide 'pure-irc-dcc-file)

;;; pure-irc-dcc-file.el ends here
