;;; pure-vs.el --- Generic function and loader for PURE version-string module

;; Copyright (C) 2000 by Project Pure.

;; Author: SHIMADA Mitsunobu <simm@irc.fan.gr.jp>
;; Keywords: PURE, version-string

;; $Id: pure-vs.el,v 1.4 2000/12/14 15:05:10 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; PURE means "Primitive Universal Relay-chat Environment."
;; VS means version-string

;;; Code:

(setq load-path (cons (expand-file-name ".") load-path))

(defconst pure-version-major 0
  "Major version number of PURE current version")
;(defconst pure-version-minor 0
;  "Minor version number of PURE current version")
(defconst pure-version-branch 3
  "Branch number of PURE current version")
(defconst pure-version-cvs nil
  "If non-nil, this version of PURE is CVS version.
If nil, package version.")
(defconst pure-version-string
  (if (boundp 'pure-version-minor)
      (format "%d.%d.%d%c"
	      pure-version-major pure-version-minor pure-version-branch
	      (if pure-version-cvs ?c ?p))
    (format "%d.%d%c"
	    pure-version-major pure-version-branch
	    (if pure-version-cvs ?c ?p)))
  "Version string of PURE current version")

(cond ((featurep 'xemacs)
       (require 'pure-vs-xmas))
      ((and (boundp 'emacs-major-version) (<= 20 emacs-major-version))
       (require 'pure-vs-e20))
      ((featurep 'mule)
       (require 'pure-vs-mule2))
      ((boundp 'NEMACS)
       (require 'pure-vs-nemacs))
      (t (require 'pure-vs-se)))

(defvar pure-vs-make-hook nil
  "Hook for pure-vs-make. This is used in Rail library.")

(defun pure-vs-make-emacs ()
  "Make Emacs version string (internal)"
  (run-hooks 'pure-vs-make-hook)
  pure-vs-emacs)

(defun pure-vs-make (&optional client)
  "Make version string (internal)"
  (run-hooks 'pure-vs-make-hook)
  (if (stringp client)
      (format "%s Pure/%s on %s" client pure-version-string pure-vs-emacs)
    (format "Pure/%s on %s" pure-version-string pure-vs-emacs)))

;; That's all
(provide 'pure-vs)

;;; pure-vs.el ends here
