;;; make.el --- Make file for PURE

;; Copyright (C) 2000 by Project Pure.

;; Author: SHIMADA Mitsunobu <simm@pure.fan.gr.jp>
;; Keywords: PURE, make, install

;; $Id: make.el,v 1.12 2001/09/16 15:30:24 simm Exp $

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:

(require 'pure-make)

(defvar make-pure-lispfile-list
      '("pure-make.el"
	"pure-generic.el"
	"pure-bl.el"
	"pure-ds.el"
	"pure-cs.el"
	"pure-cs-japanese.el"
	("pure-cs-e20.el"    . pure-make-emacs20-p)
	("pure-cs-mule2.el"  . pure-make-mule2-p)
	("pure-cs-nemacs.el" . pure-make-nemacs-p)
	("pure-cs-se.el"     . pure-make-sbemacs-p)
	("pure-cs-xmas.el"   . pure-make-xemacs-p)
	"pure-irc-send.el"
	"pure-irc-dcc.el"
	"pure-irc-dcc-info.el"
	"pure-pr-dcc-pure.el"
	"pure-pr-dcc-irchat.el"
	"pure-pr-filter.el"
	"pure-pr-ipaddr.el"
	"pure-pr-server.el"
	"pure-vs.el"))

(defun make-pure-bindir (&optional default)
  (let ((bindir (or (pure-make-get-argument 'bindir) default pure-make-bindir)))
    (if (stringp bindir) (pure-make-directory bindir))
    bindir))

(defun make-pure-compile ()
  (pure-make-compile-lisp make-pure-lispfile-list 'pure-cs 'pure-vs))

(defun make-pure-install ()
  (make-pure-compile)
  (pure-make-install-current-lisp
   make-pure-lispfile-list
   ;;(or (car command-line-args-left) "pure"))
   (or (pure-make-get-argument 'lispdir) "pure"))
  (message "Install binary files...")
  (mapcar
   (function
    (lambda (item)
      (if (file-exists-p (concat "c/" item pure-make-exec-ext))
	  (progn
	    (copy-file (concat "c/" item pure-make-exec-ext)
		       (expand-file-name (concat item pure-make-exec-ext) (make-pure-bindir))
		       t t)
	    (message "%s%s -> %s%s"
		     item pure-make-exec-ext
		     (expand-file-name item (make-pure-bindir)) pure-make-exec-ext)))))
   '("puresv" "dccput" "dccget")))

(defun make-pure-package ()
  (if (pure-make-get-argument 'lispdir)
      (make-pure-install)
    (make-pure-compile)
    (pure-make-install-package
     make-pure-lispfile-list
     "pure"
     (or (pure-make-get-argument 'packagedir) pure-make-package-base)
     (cons (cons (concat "puresv" pure-make-exec-ext) "c") (make-pure-bindir 'etcdir))
     (cons (cons (concat "dccput" pure-make-exec-ext) "c") (make-pure-bindir 'etcdir))
     (cons (cons (concat "dccget" pure-make-exec-ext) "c") (make-pure-bindir 'etcdir)))))

;; That's all
(provide 'make)

;;; make.el ends here
